"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveWithLocationRedirectionHandling = exports.isLocationRedirectionError = void 0;
const driver_definitions_1 = require("@fluidframework/driver-definitions");
const telemetry_utils_1 = require("@fluidframework/telemetry-utils");
/**
 * Checks if the error is location redirection error.
 * @param error - error whose type is to be determined.
 * @returns `true` is the error is location redirection error, otherwise `false`.
 */
function isLocationRedirectionError(error) {
    return (typeof error === "object" &&
        error !== null &&
        error.errorType === driver_definitions_1.DriverErrorTypes.locationRedirection);
}
exports.isLocationRedirectionError = isLocationRedirectionError;
/**
 * Handles location redirection while fulfilling the loader request.
 * @param api - Callback in which user can wrap the loader.resolve or loader.request call.
 * @param request - request to be resolved.
 * @param urlResolver - resolver used to resolve the url.
 * @param logger - logger to send events.
 * @returns Response from the API call.
 */
async function resolveWithLocationRedirectionHandling(api, request, urlResolver, logger) {
    let req = request;
    const childLogger = (0, telemetry_utils_1.createChildLogger)({ logger, namespace: "LocationRedirection" });
    for (;;) {
        try {
            return await api(req);
        }
        catch (error) {
            if (!isLocationRedirectionError(error)) {
                throw error;
            }
            childLogger.sendTelemetryEvent({ eventName: "LocationRedirectionError" });
            const resolvedUrl = error.redirectUrl;
            // Generate the new request with new location details from the resolved url. For datastore/relative path,
            // we don't need to pass "/" as host could have asked for a specific data store. So driver need to
            // extract it from the resolved url.
            const absoluteUrl = await urlResolver.getAbsoluteUrl(resolvedUrl, "", undefined);
            req = { url: absoluteUrl, headers: req.headers };
        }
    }
}
exports.resolveWithLocationRedirectionHandling = resolveWithLocationRedirectionHandling;
//# sourceMappingURL=resolveWithLocationRedirection.js.map